package com.vendor.loader.util;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.loader.Log.Log;
import com.vendor.loader.Messages;
import com.vendor.loader.controler.Builder;
import com.vendor.loader.controler.GlobleControler;
import com.vendor.loader.struct.UpgradeFileInfo;
import com.vendor.loader.struct.UpgradeInfo;
import com.vendor.loader.struct.UsbHeader;
import com.vendor.loader.struct.UsbImgMap;
import com.vendor.loader.struct.UsbManuInfo;
import com.vendor.loader.struct.ssu.SsuUpgradeInfo;
import com.vendor.loader.util.Crc32;
import com.vendor.loader.util.FileHelper;
import com.vendor.loader.util.FileUtil;
import com.vendor.loader.util.LoaderException;
import com.vendor.loader.util.PropertiesUtility;
import com.vendor.loader.util.ssu.SsuPackageOptions;
import com.vendor.loader.views.LoaderView;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;

public class SOCTUsbPackageBuilder implements Builder {
  private static final int NumberOfManufacturer = 1;
  
  private static final long DownloadType = 1L;
  
  private static boolean is32Bit;
  
  private static int SHA256_BIT = 32;
  
  private static int HEAD_OFFSET = 8;
  
  private static final int BUFFER_SIZE = 262144;
  
  private static boolean is64Bit;
  
  public static void setIs32Bit(boolean is32Bit) {
    com.vendor.loader.util.SOCTUsbPackageBuilder.is32Bit = is32Bit;
  }
  
  public static void set64Bit(boolean is64Bit) {
    com.vendor.loader.util.SOCTUsbPackageBuilder.is64Bit = is64Bit;
  }
  
  public IStatus createPackage(List<UpgradeFileInfo> files, UpgradeInfo info, String file, IProgressMonitor monitor) throws IOException, LoaderException {
    monitor.setTaskName("Validate upgrade file list..");
    setIs32Bit(GlobleControler.intance.is32Bit());
    set64Bit(GlobleControler.intance.is64Bit());
    log("(SOCT Usb package) start package...");
    log("(SOCT Usb package) is32Bit..." + is32Bit);
    log("(SOCT Usb package) is64Bit..." + is64Bit);
    boolean success = FileHelper.validateUpdateFilesList(files, 503316480L);
    log("Validate result:\t" + success);
    if (!success)
      return Status.CANCEL_STATUS; 
    log("Analysis of the part file");
    monitor.setTaskName("Analysis of the part file");
    files = FileHelper.getRealBurnFileList(files, 503316480L);
    if (256 < files.size())
      throw new LoaderException(String.format(Messages.LoaderComposite_MaxPartFile, new Object[] { Integer.valueOf(256) })); 
    if (monitor.isCanceled()) {
      FileUtil.deleteFile(file);
      return Status.CANCEL_STATUS;
    } 
    int partNum = files.size();
    monitor.beginTask("Package Usb File..", 100 * partNum + 120);
    UsbHeader header = new UsbHeader();
    header.setU32MagicNum(1280262468);
    header.setU16ManuNum(1);
    int size = header.astManuInfo.length;
    for (int i = 0; i < size; i++)
      header.astManuInfo[i] = new UsbManuInfo(); 
    UsbManuInfo usbManuInfo = header.astManuInfo[0];
    monitor.worked(5);
    usbManuInfo.setU32ManuId(info.getManufacturerId());
    usbManuInfo.setU32HardVersion(info.getHardwareVersion());
    usbManuInfo.setU32SoftVersion(info.getSoftwareVersion());
    monitor.worked(5);
    usbManuInfo.setU32SnStart(info.getStartSn());
    usbManuInfo.setU32SnEnd(info.getEndSn());
    usbManuInfo.setU32DownloadType(1);
    if (LoaderView.isChipBranchSeries1()) {
      usbManuInfo.setU16InternalVersion((short)1);
    } else {
      usbManuInfo.setU16InternalVersion((short)0);
    } 
    usbManuInfo.setU16Reserved((short)0);
    usbManuInfo.setU16ImgMapNum((short)partNum);
    if (monitor.isCanceled()) {
      FileUtil.deleteFile(file);
      return Status.CANCEL_STATUS;
    } 
    monitor.worked(10);
    for (int j = 0; j < partNum; j++) {
      if (monitor.isCanceled()) {
        FileUtil.deleteFile(file);
        return Status.CANCEL_STATUS;
      } 
      monitor.worked(100);
      UpgradeFileInfo f = files.get(j);
      UsbImgMap pImg = new UsbImgMap();
      usbManuInfo.getAstImgMap()[j] = pImg;
      pImg.setU64FileLength(f.getFileLength());
      pImg.setU64FlashAddr(f.getStartAddress());
      pImg.setU64Partition_endaddr(f.getEndAddress());
      pImg.setUFlashType(f.getFlashType());
      pImg.setlFlashIndex(f.getFlashIndex());
      pImg.setStrFileName(f.getFileName());
      pImg.setU8PartitionName(f.getPartitionName());
      pImg.setU32PartitionVersion(f.getPartitionVersion());
      pImg.setU32PartitionVerifyTag(f.getPartitionVerifyTag());
    } 
    header.setU32HeaderLen(getHeaderLen(header));
    header.setU64FileLen(calcOffset(header));
    return createUsbPackage(header, file, new SubProgressMonitor(monitor, 100, 4));
  }
  
  private static IStatus createUsbPackage(UsbHeader header, String file, SubProgressMonitor monitor) throws IOException, LoaderException {
    if (monitor.isCanceled()) {
      FileUtil.deleteFile(file);
      return Status.CANCEL_STATUS;
    } 
    FileOutputStream out = null;
    RandomAccessFile raf = null;
    ByteArrayOutputStream temp = null;
    try {
      out = new FileOutputStream(file);
      temp = new ByteArrayOutputStream(header.getU32HeaderLen());
      writeDword(temp, header.getU32MagicNum());
      log("[UsbHeader][u32MagicNum]=" + header.getU32MagicNum());
      writeDword(temp, 0L);
      writeDword(temp, header.getU32HeaderLen());
      log("[UsbHeader][u32HeaderLen]=" + header.getU32HeaderLen());
      if (LoaderView.isChipBranchSeries1()) {
        writeLongword(temp, header.getU64FileLen());
      } else if (is64Bit || is32Bit) {
        writeDword(temp, (int)header.getU64FileLen());
      } else {
        writeLongword(temp, header.getU64FileLen());
      } 
      log("[UsbHeader][u64FileLen]=" + header.getU64FileLen());
      writeWordInt(temp, header.getU16ManuNum());
      log("[UsbHeader][u16ManuNum]=" + header.getU16ManuNum());
      monitor.beginTask("", header.getU16ManuNum());
      for (int i = 0, l = header.getU16ManuNum(); i < l; i++) {
        if (monitor.isCanceled()) {
          FileUtil.deleteFile(file);
          return Status.CANCEL_STATUS;
        } 
        UsbManuInfo pManu = header.astManuInfo[i];
        writeDword(temp, pManu.getU32ManuId());
        log("[UsbManuInfo][u32ManuId]=" + pManu.getU32ManuId());
        writeDword(temp, pManu.getU32HardVersion());
        log("[UsbManuInfo][u32HardVersion]=" + pManu.getU32HardVersion());
        writeDword(temp, pManu.getU32SoftVersion());
        log("[UsbManuInfo][u32SoftVersion]=" + pManu.getU32SoftVersion());
        writeDword(temp, pManu.getU32SnStart());
        log("[UsbManuInfo][u32SnStart]=" + pManu.getU32SnStart());
        writeDword(temp, pManu.getU32SnEnd());
        log("[UsbManuInfo][u32SnEnd]=" + pManu.getU32SnEnd());
        writeDword(temp, pManu.getU32DownloadType());
        log("[UsbManuInfo][u32DownloadType]=" + pManu.getU32DownloadType());
        writeWordInt(temp, pManu.getU16InternalVersion());
        log("[UsbManuInfo][u16InternalVersion]=" + pManu.getU16InternalVersion());
        writeWordInt(temp, pManu.getU16Reserved());
        log("[UsbManuInfo][u16Reserved]=" + pManu.getU16Reserved());
        writeWordInt(temp, pManu.getU16ImgMapNum());
        log("[UsbManuInfo][u16ImgMapNum]=" + pManu.getU16ImgMapNum());
        for (int j = 0, k = pManu.getU16ImgMapNum(); j < k; j++) {
          if (monitor.isCanceled()) {
            FileUtil.deleteFile(file);
            return Status.CANCEL_STATUS;
          } 
          UsbImgMap pImg = pManu.getAstImgMap()[j];
          log("File=" + pImg.getStrFileName());
          if (LoaderView.isChipBranchSeries1()) {
            writeLongword(temp, pImg.getU64FileLength());
            log("[UsbImgMap][u64FileLength]=" + pImg.getU64FileLength());
            writeLongword(temp, pImg.getU64Offset());
            log("[UsbImgMap][u64Offset]=" + pImg.getU64Offset());
          } else {
            writeDword(temp, (int)pImg.getU64FileLength());
            log("[UsbImgMap][u32FileLength]=" + pImg.getU64FileLength());
            writeDword(temp, (int)pImg.getU64Offset());
            log("[UsbImgMap][u32Offset]=" + pImg.getU64Offset());
          } 
          if (is32Bit) {
            writeDword(temp, (int)pImg.getU64FlashAddr());
            log("[UsbImgMap][u32FlashAddr]=" + pImg.getU64FlashAddr());
            writeDword(temp, (int)pImg.getU64Partition_endaddr());
            log("[UsbImgMap][u32Partition_endaddr]=" + pImg.getU64Partition_endaddr());
          } else {
            writeLongword(temp, pImg.getU64FlashAddr());
            log("[UsbImgMap][u64FlashAddr]=" + pImg.getU64FlashAddr());
            writeLongword(temp, pImg.getU64Partition_endaddr());
            log("[UsbImgMap][u64Partition_endaddr]=" + pImg.getU64Partition_endaddr());
          } 
          writeDword(temp, pImg.getUFlashType().ordinal());
          log("[UsbImgMap][uFlashType]=" + pImg.getUFlashType());
          writeDword(temp, pImg.getlFlashIndex());
          if (is64Bit) {
            writeDword(temp, pImg.getU32PartitionVersion());
            writeDwordLow8Bit(temp, pImg.getU32PartitionVerifyTag());
            String name = pImg.getU8PartitionName();
            if (name == null || name.isEmpty()) {
              writeWordInt(temp, 0);
            } else {
              byte[] namas = name.getBytes(System.getProperty("file.encoding"));
              writeWordInt(temp, name.length());
              writeDword(temp, namas);
            } 
            writeWordInt(temp, 0);
          } 
        } 
        if (monitor.isCanceled()) {
          FileUtil.deleteFile(file);
          return Status.CANCEL_STATUS;
        } 
        byte[] bytes = temp.toByteArray();
        long crc = 0L;
        if (LoaderView.isChipBranchSeries2()) {
          byte[] bytesTemp = new byte[bytes.length - HEAD_OFFSET];
          System.arraycopy(bytes, HEAD_OFFSET, bytesTemp, 0, bytes.length - HEAD_OFFSET);
          crc = sha256(bytesTemp);
        } else {
          crc = Crc32.compute(bytes, header.getU32HeaderLen());
        } 
        Crc32.updateCrcField(bytes, 4, crc);
        monitor.setTaskName("Write Head");
        writeToFile(out, bytes);
        log("Write dst file image");
        for (int m = 0, n = pManu.getU16ImgMapNum(); m < n; m++) {
          if (monitor.isCanceled()) {
            FileUtil.deleteFile(file);
            return Status.CANCEL_STATUS;
          } 
          UsbImgMap pImg = pManu.getAstImgMap()[m];
          File f = new File(pImg.getStrFileName());
          long fileLength = f.length();
          if (LoaderView.isChipBranchSeries1()) {
            writeLongword(out, fileLength);
          } else {
            writeDword(out, fileLength);
          } 
          log("Write file length = " + f.getName() + "\t" + f.length());
          if (LoaderView.isChipBranchSeries2()) {
            crc = sha256FileCompute(pImg.getStrFileName());
          } else {
            crc = Crc32.compute(pImg.getStrFileName());
          } 
          log("Write crc = " + crc);
          writeDword(out, crc);
          writeToFile(out, f);
        } 
        if (monitor.isCanceled()) {
          FileUtil.deleteFile(file);
          return Status.CANCEL_STATUS;
        } 
        monitor.worked(1);
      } 
      if (!LoaderView.isChipBranchSeries0()) {
        int tmpLen = (int)(header.getU64FileLen() - calcOffset(header));
        raf = new RandomAccessFile(file, "rw");
        raf.seek(calcOffset(header));
        byte[] buffer = new byte[tmpLen];
        for (int j = 0; j < buffer.length; j++)
          buffer[j] = -1; 
        raf.write(buffer);
      } 
    } catch (IOException e) {
      throw new LoaderException(Messages.UsbPackageBuilder_CreateError);
    } finally {
      closeResourse(out, raf, temp);
      if (monitor.isCanceled())
        FileUtil.deleteFile(file); 
    } 
    return Status.OK_STATUS;
  }
  
  private static long sha256FileCompute(String filename) throws IOException {
    long crc = 0L;
    File f = new File(filename);
    FileInputStream in = null;
    try {
      in = new FileInputStream(f);
      long length = f.length();
      byte[] buffer = new byte[262144];
      MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
      while (length > 0L) {
        int len = in.read(buffer);
        length -= len;
        messageDigest.update(buffer, 0, len);
      } 
      byte[] hash = messageDigest.digest();
      crc = (hash[SHA256_BIT - 4] & 0xFF | (hash[SHA256_BIT - 3] & 0xFF) << 8 | (hash[SHA256_BIT - 2] & 0xFF) << 16 | (hash[SHA256_BIT - 1] & 0xFF) << 24);
    } catch (NoSuchAlgorithmException noSuchAlgorithmException) {
    
    } finally {
      if (in != null)
        try {
          in.close();
        } catch (Exception exception) {} 
    } 
    return crc;
  }
  
  private static long sha256(byte[] bytes) {
    long crc = 0L;
    try {
      MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
      messageDigest.update(bytes);
      byte[] byteBuffer = messageDigest.digest();
      if (byteBuffer.length < SHA256_BIT)
        return crc; 
      crc = (byteBuffer[SHA256_BIT - 4] & 0xFF | (byteBuffer[SHA256_BIT - 3] & 0xFF) << 8 | (byteBuffer[SHA256_BIT - 2] & 0xFF) << 16 | (byteBuffer[SHA256_BIT - 1] & 0xFF) << 24);
    } catch (NoSuchAlgorithmException noSuchAlgorithmException) {}
    return crc;
  }
  
  private static void closeResourse(FileOutputStream out, RandomAccessFile raf, ByteArrayOutputStream temp) throws LoaderException {
    int exceptionNumber = 0;
    try {
      if (out != null)
        out.close(); 
    } catch (IOException e1) {
      exceptionNumber++;
    } 
    try {
      if (raf != null)
        raf.close(); 
    } catch (IOException e1) {
      exceptionNumber++;
    } 
    try {
      if (temp != null)
        temp.close(); 
    } catch (IOException e1) {
      exceptionNumber++;
    } 
    if (exceptionNumber > 0)
      throw new LoaderException(Messages.UsbPackageBuilder_WriteError); 
  }
  
  private static void writeDword(ByteArrayOutputStream temp, byte[] PartitionNameS) {
    try {
      temp.write(PartitionNameS);
    } catch (IOException iOException) {}
  }
  
  private static void writeToFile(FileOutputStream out, File f) throws IOException {
    FileInputStream inputStream = null;
    try {
      inputStream = new FileInputStream(f);
      byte[] buffer = new byte[1048576];
      long length = f.length();
      while (length > 0L) {
        int read = inputStream.read(buffer);
        out.write(buffer, 0, read);
        length -= read;
      } 
    } finally {
      if (inputStream != null)
        try {
          inputStream.close();
        } catch (Exception exception) {} 
    } 
  }
  
  private static void writeToFile(FileOutputStream out, byte[] buf) throws LoaderException {
    try {
      out.write(buf);
      out.flush();
    } catch (Exception e1) {
      throw new LoaderException(Messages.UsbPackageBuilder_WriteError);
    } 
  }
  
  private static long calcOffset(UsbHeader header) {
    long offset = header.getU32HeaderLen();
    for (int i = 0, l = header.getU16ManuNum(); i < l; i++) {
      for (int j = 0, k = header.astManuInfo[i].getU16ImgMapNum(); j < k; j++) {
        UsbImgMap pImg = header.astManuInfo[i].getAstImgMap()[j];
        pImg.setU64Offset(offset);
        if (LoaderView.isChipBranchSeries1()) {
          offset += 12L;
        } else {
          offset += 8L;
        } 
        offset += pImg.getU64FileLength();
      } 
    } 
    return offset;
  }
  
  public static int getMenuInfoLen(UsbManuInfo pManu) {
    int img_len = 0;
    int manu_len = 30;
    if (is32Bit) {
      img_len = 24;
    } else if (LoaderView.isChipBranchSeries1()) {
      img_len = 40;
    } else {
      img_len = 32;
    } 
    if (is64Bit) {
      int length = 0;
      for (int j = 0, k = pManu.getU16ImgMapNum(); j < k; j++) {
        UsbImgMap pImg = pManu.getAstImgMap()[j];
        String name = pImg.getU8PartitionName();
        if (name == null || name.isEmpty()) {
          length += img_len + 4 + 4 + 2 + 2;
        } else {
          length += img_len + 4 + 4 + 2 + pImg.getU8PartitionName().length() + 2;
        } 
      } 
      return manu_len + length;
    } 
    return manu_len + img_len * pManu.getU16ImgMapNum();
  }
  
  public static int getHeaderLen(UsbHeader header) {
    int len = 0;
    if (LoaderView.isChipBranchSeries1()) {
      len = 22;
    } else if (is64Bit || is32Bit) {
      len = 18;
    } else {
      len = 22;
    } 
    if (header.getU16ManuNum() > 3)
      header.setU16ManuNum(0); 
    for (int i = 0, l = header.getU16ManuNum(); i < l; i++)
      len += getMenuInfoLen(header.astManuInfo[i]); 
    return len;
  }
  
  public static void writeLongword(OutputStream out, long val) throws IOException {
    out.write((byte)(int)(val >> 56L) & 0xFF);
    out.write((byte)(int)(val >> 48L) & 0xFF);
    out.write((byte)(int)(val >> 40L) & 0xFF);
    out.write((byte)(int)(val >> 32L) & 0xFF);
    out.write((byte)(int)(val >> 24L) & 0xFF);
    out.write((byte)(int)(val >> 16L) & 0xFF);
    out.write((byte)(int)(val >> 8L) & 0xFF);
    out.write((byte)(int)(val & 0xFFL));
  }
  
  public static void writeDword(OutputStream out, long val) throws IOException {
    out.write((int)(val >>> 24L & 0xFFL));
    out.write((int)(val >>> 16L & 0xFFL));
    out.write((int)(val >>> 8L & 0xFFL));
    out.write((int)(val & 0xFFL));
  }
  
  public static void writeDwordLow8Bit(OutputStream out, long val) throws IOException {
    out.write(0);
    out.write(0);
    out.write(0);
    out.write((int)(val & 0xFFL));
  }
  
  public static void writeWordInt(OutputStream out, int val) throws IOException {
    out.write(val >>> 8 & 0xFF);
    out.write(val & 0xFF);
  }
  
  public IStatus createPackage(List<UpgradeFileInfo> fileInfoList, SsuUpgradeInfo info, SsuPackageOptions packageOtions, String tsFileName, IProgressMonitor monitor) throws IOException, LoaderException {
    return null;
  }
  
  private static void log(String infoStr) {
    if (PropertiesUtility.isCmdLine())
      Log.println(infoStr); 
  }
}
